# -*- coding: utf-8 -*-
from __future__ import with_statement, absolute_import, unicode_literals

import glob
import os.path as op
import re

import click

from ..context import pass_context, on_finished, on_started


def get_image_id(events):
    image_id = None
    for event in events:
        if 'stream' in event:
            match = re.search(r'Successfully built ([0-9a-f]+)',
                              event.get('stream', ''))
            if match:
                image_id = match.group(1)
    if image_id is None:
        raise click.ClickException("Cannot get image id")
    return image_id


@click.command('build')
@click.option('--force-rm', is_flag=True, default=False,
              help="Always remove intermediate containers, even after "
                   "unsuccessful builds")
@click.option('--no-cache', is_flag=True, default=False,
              help="Do not use cache when building the image")
@click.option('--pull', is_flag=True, default=False,
              help="Attempt to pull a newer version of the image")
@click.option('-q', '--quiet', is_flag=True, default=False,
              help="Suppress the verbose output generated by the containers")
@click.option('--rm/--no-rm', is_flag=True, default=True,
              help="Remove intermediate containers after a successful build")
@pass_context
@on_finished(lambda ctx: ctx.state.dump())
@on_finished("clean")
@on_started("stop")
@on_started(lambda ctx: ctx.assert_valid_env())
def cli(ctx, force_rm, no_cache, pull, quiet, rm):
    """Build base images"""
    ctx.log('Starting oardocker build')
    dockerfiles = glob.glob(op.join(ctx.envdir, "images", "*", "Dockerfile"))
    dockerfiles.sort()

    def get_prefix_width():
        for dockerfile in dockerfiles:
            yield len(op.basename(op.dirname(dockerfile)))
    max_prefix_width = max(get_prefix_width())
    for dockerfile in dockerfiles:
        dirname = op.dirname(dockerfile)
        name = op.basename(dirname)
        if name in ("frontend", "node", "server"):
            tag = "%s:base" % ctx.image_name(name)
        else:
            tag = "%s:latest" % ctx.image_name(name)
        padding = ' ' * (max_prefix_width - len(name))
        prefix = click.style(''.join([padding, name, ' | ']), fg="green")
        args = ["build"]
        if force_rm:
            args.append("--force-rm")
        if no_cache:
            args.append("--no-cache")
        if pull:
            args.append("--pull")
        if quiet:
            args.append("--quiet")
        if rm:
            args.append("--rm")
        args.extend(["--tag", tag])
        args.append(dirname)
        for line in ctx.docker.cli(args, _iter=True):
            ctx.log(prefix + line, nl=False)
        ctx.state["images"].append(tag)
